/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : MicroBoltAccurateDelays.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 12/30/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Main C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include "MicroBoltAccurateDelays.h"

/*
|------------------------------------------------------------------------------------------
|     Function        :   main
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Main function for system
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void main(void)
  {

  __DISABLE_INTERRUPT();                        // Disable all interrupts

/*
|-------------------------
| Config MAM 
|-------------------------
*/
  MAM_CR = 0x00;                                // Turn MAM off (default)
  MAM_TIM = 0x04;                               // Set flash timing to 4 clock cycles

  MAM_CR = 0x02;                                // Fully enable the Memory Accleration Module

/*
|-------------------------
| Config PLL and CCLK 
|-------------------------
*/
  SCB_PLLCFG |= 0x23;				 	   		            // Set to 59 MHz (0x03 is multiply value of 4)
  SCB_PLLCON |= 0x01;                           // Enable the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG

/*
|-------------------------
| Config PCLK 
|-------------------------
*/
  SCB_VPBDIV = 0;                               // Peripheral clock is 1/4th Processor clock which equals 14.7456 MHz

/*
|-------------------------
| Configure VIC 
|-------------------------
*/
  VICVectAddr0 = (unsigned)pll_isr;             // Assign the PLL lock ISR vector address
  VICVectCntl0 = INTERRUPT_CHANNEL_FOR_PLL;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_PLL;      // Enable the interrupt  

/*
|-------------------------
| Config GPIO 
|-------------------------
*/
  PCB_PINSEL0=0x00000000;                         
  PCB_PINSEL1=0x55400000;                       // Secondary JTAG pins 

  GPIO_IODIR |= MICROBOLT_LED;                  // Setup MicroBolt LED as output
  GPIO_IODIR |= P0_9;                           // Setup P0_9 as output

  GPIO_IOSET = MICROBOLT_LED;                   // Turn on MicroBolt LED

  GPIO_IOCLR=0xffffffff;                        // Clear all pins to start with

/*
|----------------------------------------------------
|     Start of application
|----------------------------------------------------
*/
  while(1)                                      // Do this forever
    {
    GPIO_IOSET = MICROBOLT_LED;                 // Turn on MicroBolt LED

    GPIO_IOSET = P0_9;                          // Measure delay via P0_9 On time (LPC2106 pin toggle takes about 1.4 uS)
    Delay10uS();                                // 10 uS Delay
    GPIO_IOCLR = P0_9;
    Delay10uS();

    GPIO_IOSET = P0_9;                          
    Delay100uS();                               // 100 uS Delay
    GPIO_IOCLR = P0_9;
    Delay100uS();

    GPIO_IOSET = P0_9;                          
    Delay1mS();                                 // 1 mS Delay
    GPIO_IOCLR = P0_9;
    Delay1mS();

    GPIO_IOSET = P0_9;                          
    Delay10mS();                                // 10 mS Delay
    GPIO_IOCLR = P0_9;    
    Delay10mS();

    GPIO_IOSET = P0_9;                          
    Delay100mS();                               // 100 mS Delay
    GPIO_IOCLR = P0_9;
    Delay100mS();

    GPIO_IOCLR = MICROBOLT_LED;                 // Turn Off MicroBolt LED

    GPIO_IOSET = P0_9;                          
    Delay1Sec();                                // 1 Second Delay
    GPIO_IOCLR = P0_9;
    Delay1Sec();
    }
  }

/*
|----------------------------------------------------
|     Functions
|----------------------------------------------------
*/ 

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay10uS
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 10 uS when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void Delay10uS(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 7; Delay++);          // Delay for 10 uS including function call
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay100uS
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 100 uS when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void Delay100uS(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 109; Delay++);         // Delay for 100 uS including function call     
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay1mS
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 1 mS when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void Delay1mS(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 1000; Delay++);       // Delay for 1 mS including function call
  for (Delay = 0; Delay < 100; Delay++);       
  for (Delay = 0; Delay < 30; Delay++); 
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay10mS
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 10 mS when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void Delay10mS(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 9200; Delay++);       // Delay for 10 mS including function call
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay100mS
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 100 mS when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void Delay100mS(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 92000; Delay++);      // Delay for 100 mS including function call
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Delay1Sec
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Delay for 1 Second when PLLCFG is set to 0x23 (58.98 MHz) 
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
void Delay1Sec(void)   
  {
	volatile unsigned int Delay;

  for (Delay = 0; Delay < 920000; Delay++);     // Delay for 1 Second including function call
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   pll_isr
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Once PLL has locked, connect it and use for system clock
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/
#pragma interrupt_handler pll_isr

void pll_isr(void)
  {
  SCB_PLLCON |= 0x02;                           // Connect the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG
  VICIntEnClear = PLL_CLR;                      // Clear the PLL interrupt
  VICVectAddr = VIC_ACK;                        // Acknowledge Interrupt
  }


